@@ -183,6 +183,10 @@ static void babel_read_protocol(struct event *thread)
             flog_err_sys(EC_LIB_SOCKET, "recv: %s", safe_strerror(errno));
         }
     } else {
+        if(ntohs(sin6.sin6_port) != BABEL_PORT) {
+            return;
+        }
+
         FOR_ALL_INTERFACES(vrf, ifp) {
             if(!if_up(ifp))
                 continue;
@@ -204,7 +208,7 @@ static void babel_read_protocol(struct event *thread)
  making these inits have sense. */
 static void babel_init_routing_process(struct event *thread)
 {
-    myseqno = (frr_weak_random() & 0xFFFF);
+    myseqno = CHECK_FLAG(frr_weak_random(), 0xFFFF);
     babel_get_myid();
     babel_load_state_file();
     debugf(BABEL_DEBUG_COMMON, "My ID is : %s.", format_eui64(myid));
@@ -242,7 +246,7 @@ babel_get_myid(void)
     /* We failed to get a global EUI64 from the interfaces we were given.
      Let's try to find an interface with a MAC address. */
     for(i = 1; i < 256; i++) {
-        char buf[INTERFACE_NAMSIZ], *ifname;
+        char buf[IFNAMSIZ], *ifname;
         unsigned char eui[8];
         ifname = if_indextoname(i, buf);
         if(ifname == NULL)
@@ -299,8 +303,7 @@ babel_initial_noise(void)
 }
 
 /* Delete all the added babel routes, make babeld only speak to zebra. */
-static void
-babel_clean_routing_process(void)
+void babel_clean_routing_process(void)
 {
     flush_all_routes();
     babel_interface_close_all();
@@ -444,8 +447,8 @@ babel_fill_with_next_timeout(struct timeval *tv)
 #if (defined NO_DEBUG)
 #define printIfMin(a,b,c,d)
 #else
-#define printIfMin(a, b, c, d)                                                 \
-	if (unlikely(debug & BABEL_DEBUG_TIMEOUT)) {                           \
+#define printIfMin(a, b, c, d)                                         \
+	if (unlikely(CHECK_FLAG(debug, BABEL_DEBUG_TIMEOUT))) {            \
 		printIfMin(a, b, c, d);                                        \
 	}
 
@@ -696,9 +699,8 @@ DEFPY (babel_set_smoothing_half_life,
 
 DEFUN (babel_distribute_list,
        babel_distribute_list_cmd,
-       "distribute-list [prefix] ACCESSLIST4_NAME <in|out> [WORD]",
+       "distribute-list ACCESSLIST4_NAME <in|out> [WORD]",
        "Filter networks in routing updates\n"
-       "Specify a prefix\n"
        "Access-list name\n"
        "Filter incoming routing updates\n"
        "Filter outgoing routing updates\n"
@@ -710,16 +712,26 @@ DEFUN (babel_distribute_list,
 	if (argv[argc - 1]->type == VARIABLE_TKN)
 		ifname = argv[argc - 1]->arg;
 
-	return distribute_list_parser(prefix, true, argv[2 + prefix]->text,
+	return distribute_list_parser(babel_routing_process->distribute_ctx,
+				      prefix, true, argv[2 + prefix]->text,
 				      argv[1 + prefix]->arg, ifname);
 }
 
+ALIAS (babel_distribute_list,
+       babel_distribute_list_prefix_cmd,
+       "distribute-list prefix PREFIXLIST4_NAME <in|out> [WORD]",
+       "Filter networks in routing updates\n"
+       "Specify a prefix list\n"
+       "Prefix-list name\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n"
+       "Interface name\n")
+
 DEFUN (babel_no_distribute_list,
        babel_no_distribute_list_cmd,
-       "no distribute-list [prefix] ACCESSLIST4_NAME <in|out> [WORD]",
+       "no distribute-list ACCESSLIST4_NAME <in|out> [WORD]",
        NO_STR
        "Filter networks in routing updates\n"
-       "Specify a prefix\n"
        "Access-list name\n"
        "Filter incoming routing updates\n"
        "Filter outgoing routing updates\n"
@@ -731,17 +743,28 @@ DEFUN (babel_no_distribute_list,
 	if (argv[argc - 1]->type == VARIABLE_TKN)
 		ifname = argv[argc - 1]->arg;
 
-	return distribute_list_no_parser(vty, prefix, true,
+	return distribute_list_no_parser(babel_routing_process->distribute_ctx,
+					 vty, prefix, true,
 					 argv[3 + prefix]->text,
 					 argv[2 + prefix]->arg, ifname);
 }
 
+ALIAS (babel_no_distribute_list,
+       babel_no_distribute_list_prefix_cmd,
+       "no distribute-list prefix PREFIXLIST4_NAME <in|out> [WORD]",
+       NO_STR
+       "Filter networks in routing updates\n"
+       "Specify a prefix list\n"
+       "Prefix-list name\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n"
+       "Interface name\n")
+
 DEFUN (babel_ipv6_distribute_list,
        babel_ipv6_distribute_list_cmd,
-       "ipv6 distribute-list [prefix] ACCESSLIST6_NAME <in|out> [WORD]",
+       "ipv6 distribute-list ACCESSLIST6_NAME <in|out> [WORD]",
        "IPv6\n"
        "Filter networks in routing updates\n"
-       "Specify a prefix\n"
        "Access-list name\n"
        "Filter incoming routing updates\n"
        "Filter outgoing routing updates\n"
@@ -753,17 +776,28 @@ DEFUN (babel_ipv6_distribute_list,
 	if (argv[argc - 1]->type == VARIABLE_TKN)
 		ifname = argv[argc - 1]->arg;
 
-	return distribute_list_parser(prefix, false, argv[3 + prefix]->text,
+	return distribute_list_parser(babel_routing_process->distribute_ctx,
+				      prefix, false, argv[3 + prefix]->text,
 				      argv[2 + prefix]->arg, ifname);
 }
 
+ALIAS (babel_ipv6_distribute_list,
+       babel_ipv6_distribute_list_prefix_cmd,
+       "ipv6 distribute-list prefix PREFIXLIST6_NAME <in|out> [WORD]",
+       "IPv6\n"
+       "Filter networks in routing updates\n"
+       "Specify a prefix list\n"
+       "Prefix-list name\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n"
+       "Interface name\n")
+
 DEFUN (babel_no_ipv6_distribute_list,
        babel_no_ipv6_distribute_list_cmd,
-       "no ipv6 distribute-list [prefix] ACCESSLIST6_NAME <in|out> [WORD]",
+       "no ipv6 distribute-list ACCESSLIST6_NAME <in|out> [WORD]",
        NO_STR
        "IPv6\n"
        "Filter networks in routing updates\n"
-       "Specify a prefix\n"
        "Access-list name\n"
        "Filter incoming routing updates\n"
        "Filter outgoing routing updates\n"
@@ -775,11 +809,24 @@ DEFUN (babel_no_ipv6_distribute_list,
 	if (argv[argc - 1]->type == VARIABLE_TKN)
 		ifname = argv[argc - 1]->arg;
 
-	return distribute_list_no_parser(vty, prefix, false,
+	return distribute_list_no_parser(babel_routing_process->distribute_ctx,
+					 vty, prefix, false,
 					 argv[4 + prefix]->text,
 					 argv[3 + prefix]->arg, ifname);
 }
 
+ALIAS (babel_no_ipv6_distribute_list,
+       babel_no_ipv6_distribute_list_prefix_cmd,
+       "no ipv6 distribute-list prefix PREFIXLIST6_NAME <in|out> [WORD]",
+       NO_STR
+       "IPv6\n"
+       "Filter networks in routing updates\n"
+       "Specify a prefix list\n"
+       "Prefix-list name\n"
+       "Filter incoming routing updates\n"
+       "Filter outgoing routing updates\n"
+       "Interface name\n")
+
 void
 babeld_quagga_init(void)
 {
@@ -797,9 +844,13 @@ babeld_quagga_init(void)
     install_element(BABEL_NODE, &babel_set_smoothing_half_life_cmd);
 
     install_element(BABEL_NODE, &babel_distribute_list_cmd);
+    install_element(BABEL_NODE, &babel_distribute_list_prefix_cmd);
     install_element(BABEL_NODE, &babel_no_distribute_list_cmd);
+    install_element(BABEL_NODE, &babel_no_distribute_list_prefix_cmd);
     install_element(BABEL_NODE, &babel_ipv6_distribute_list_cmd);
+    install_element(BABEL_NODE, &babel_ipv6_distribute_list_prefix_cmd);
     install_element(BABEL_NODE, &babel_no_ipv6_distribute_list_cmd);
+    install_element(BABEL_NODE, &babel_no_ipv6_distribute_list_prefix_cmd);
 
     vrf_cmd_init(NULL);
 
