@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /**
  * bgp_bfd.c: BGP BFD handling routines
  *
  * @copyright Copyright (C) 2015 Cumulus Networks, Inc.
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -26,7 +11,7 @@
 #include "linklist.h"
 #include "memory.h"
 #include "prefix.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "buffer.h"
 #include "stream.h"
 #include "vrf.h"
@@ -41,6 +26,7 @@
 #include "bgpd/bgp_debug.h"
 #include "bgpd/bgp_vty.h"
 #include "bgpd/bgp_packet.h"
+#include "bgpd/bgp_network.h"
 
 DEFINE_MTYPE_STATIC(BGPD, BFD_CONFIG, "BFD configuration data");
 
@@ -68,21 +54,30 @@ static void bfd_session_status_update(struct bfd_session_params *bsp,
 					peer->host);
 			return;
 		}
-		peer->last_reset = PEER_DOWN_BFD_DOWN;
 
-		/* draft-ietf-idr-bfd-subcode */
-		if (BGP_IS_VALID_STATE_FOR_NOTIF(peer->status))
-			bgp_notify_send(peer, BGP_NOTIFY_CEASE,
-					BGP_NOTIFY_CEASE_BFD_DOWN);
+		/* Once the BFD session is UP, and later BGP session is UP,
+		 * BFD notices that peer->su_local changed, and BFD session goes down.
+		 * We should trigger BGP session reset if BFD session is UP
+		 * only when BGP session is UP already.
+		 * Otherwise, we end up resetting BGP session when BFD session is UP,
+		 * when the source address is changed, e.g. 0.0.0.0 -> 10.0.0.1.
+		 */
+		if (bss->last_event > peer->uptime) {
+			peer->last_reset = PEER_DOWN_BFD_DOWN;
+			/* rfc9384 */
+			if (BGP_IS_VALID_STATE_FOR_NOTIF(peer->connection->status))
+				bgp_notify_send(peer->connection, BGP_NOTIFY_CEASE,
+						BGP_NOTIFY_CEASE_BFD_DOWN);
 
-		BGP_EVENT_ADD(peer, BGP_Stop);
+			BGP_EVENT_ADD(peer->connection, BGP_Stop);
+		}
 	}
 
-	if (bss->state == BSS_UP && bss->previous_state != BSS_UP
-	    && !peer_established(peer)) {
+	if (bss->state == BSS_UP && bss->previous_state != BSS_UP &&
+	    !peer_established(peer->connection)) {
 		if (!BGP_PEER_START_SUPPRESSED(peer)) {
-			bgp_fsm_nht_update(peer, true);
-			BGP_EVENT_ADD(peer, BGP_Start);
+			bgp_fsm_nht_update(peer->connection, peer, true);
+			BGP_EVENT_ADD(peer->connection, BGP_Start);
 		}
 	}
 }
@@ -119,6 +114,10 @@ void bgp_peer_config_apply(struct peer *p, struct peer_group *pg)
 	 */
 	gconfig = pg->conf;
 
+	if (CHECK_FLAG(gconfig->flags, PEER_FLAG_UPDATE_SOURCE) ||
+	    CHECK_FLAG(p->flags_override, PEER_FLAG_UPDATE_SOURCE))
+		bgp_peer_bfd_update_source(p);
+
 	/*
 	 * If using default control plane independent configuration,
 	 * then prefer group's (e.g. it means it wasn't manually configured).
@@ -156,58 +155,78 @@ void bgp_peer_config_apply(struct peer *p, struct peer_group *pg)
 
 void bgp_peer_bfd_update_source(struct peer *p)
 {
-	struct bfd_session_params *session = p->bfd_config->session;
-	const union sockunion *source;
+	struct bfd_session_params *session;
+	const union sockunion *source = NULL;
 	bool changed = false;
 	int family;
 	union {
 		struct in_addr v4;
 		struct in6_addr v6;
 	} src, dst;
+	struct interface *ifp;
+	union sockunion addr;
 
+	if (!p->bfd_config)
+		return;
+
+	session = p->bfd_config->session;
 	/* Nothing to do for groups. */
 	if (CHECK_FLAG(p->sflags, PEER_STATUS_GROUP))
 		return;
 
 	/* Figure out the correct source to use. */
-	if (CHECK_FLAG(p->flags, PEER_FLAG_UPDATE_SOURCE) && p->update_source)
-		source = p->update_source;
-	else
+	if (CHECK_FLAG(p->flags, PEER_FLAG_UPDATE_SOURCE)) {
+		if (p->update_source) {
+			source = p->update_source;
+		} else if (p->update_if) {
+			ifp = if_lookup_by_name(p->update_if, p->bgp->vrf_id);
+			if (ifp) {
+				sockunion_init(&addr);
+				if (bgp_update_address(ifp, &p->connection->su, &addr)) {
+					if (BGP_DEBUG(bfd, BFD_LIB))
+						zlog_debug("%s: can't find the source address for interface %s",
+							   __func__, p->update_if);
+				}
+
+				source = &addr;
+			}
+		}
+	} else {
 		source = p->su_local;
+	}
 
 	/* Update peer's source/destination addresses. */
 	bfd_sess_addresses(session, &family, &src.v6, &dst.v6);
 	if (family == AF_INET) {
-		if ((source && source->sin.sin_addr.s_addr != src.v4.s_addr)
-		    || p->su.sin.sin_addr.s_addr != dst.v4.s_addr) {
+		if ((source && source->sin.sin_addr.s_addr != src.v4.s_addr) ||
+		    p->connection->su.sin.sin_addr.s_addr != dst.v4.s_addr) {
 			if (BGP_DEBUG(bfd, BFD_LIB))
-				zlog_debug(
-					"%s: address [%pI4->%pI4] to [%pI4->%pI4]",
-					__func__, &src.v4, &dst.v4,
-					source ? &source->sin.sin_addr
-					       : &src.v4,
-					&p->su.sin.sin_addr);
+				zlog_debug("%s: address [%pI4->%pI4] to [%pI4->%pI4]",
+					   __func__, &src.v4, &dst.v4,
+					   source ? &source->sin.sin_addr
+						  : &src.v4,
+					   &p->connection->su.sin.sin_addr);
 
-			bfd_sess_set_ipv4_addrs(
-				session, source ? &source->sin.sin_addr : NULL,
-				&p->su.sin.sin_addr);
+			bfd_sess_set_ipv4_addrs(session,
+						source ? &source->sin.sin_addr
+						       : NULL,
+						&p->connection->su.sin.sin_addr);
 			changed = true;
 		}
 	} else {
-		if ((source && memcmp(&source->sin6, &src.v6, sizeof(src.v6)))
-		    || memcmp(&p->su.sin6, &dst.v6, sizeof(dst.v6))) {
+		if ((source && memcmp(&source->sin6, &src.v6, sizeof(src.v6))) ||
+		    memcmp(&p->connection->su.sin6, &dst.v6, sizeof(dst.v6))) {
 			if (BGP_DEBUG(bfd, BFD_LIB))
-				zlog_debug(
-					"%s: address [%pI6->%pI6] to [%pI6->%pI6]",
-					__func__, &src.v6, &dst.v6,
-					source ? &source->sin6.sin6_addr
-					       : &src.v6,
-					&p->su.sin6.sin6_addr);
+				zlog_debug("%s: address [%pI6->%pI6] to [%pI6->%pI6]",
+					   __func__, &src.v6, &dst.v6,
+					   source ? &source->sin6.sin6_addr
+						  : &src.v6,
+					   &p->connection->su.sin6.sin6_addr);
 
 			bfd_sess_set_ipv6_addrs(session,
 						source ? &source->sin6.sin6_addr
 						       : NULL,
-						&p->su.sin6.sin6_addr);
+						&p->connection->su.sin6.sin6_addr);
 			changed = true;
 		}
 	}
@@ -299,16 +318,17 @@ void bgp_peer_configure_bfd(struct peer *p, bool manual)
 	bgp_peer_bfd_reset(p);
 
 	/* Configure session with basic BGP peer data. */
-	if (p->su.sa.sa_family == AF_INET)
+	if (p->connection->su.sa.sa_family == AF_INET)
 		bfd_sess_set_ipv4_addrs(p->bfd_config->session,
 					p->su_local ? &p->su_local->sin.sin_addr
 						    : NULL,
-					&p->su.sin.sin_addr);
+					&p->connection->su.sin.sin_addr);
 	else
-		bfd_sess_set_ipv6_addrs(
-			p->bfd_config->session,
-			p->su_local ? &p->su_local->sin6.sin6_addr : NULL,
-			&p->su.sin6.sin6_addr);
+		bfd_sess_set_ipv6_addrs(p->bfd_config->session,
+					p->su_local
+						? &p->su_local->sin6.sin6_addr
+						: NULL,
+					&p->connection->su.sin6.sin6_addr);
 
 	bfd_sess_set_vrf(p->bfd_config->session, p->bgp->vrf_id);
 	bfd_sess_set_hop_count(p->bfd_config->session,
@@ -612,6 +632,9 @@ DEFUN(no_neighbor_bfd_profile, no_neighbor_bfd_profile_cmd,
 	if (!peer)
 		return CMD_WARNING_CONFIG_FAILED;
 
+	if (!peer->bfd_config)
+		return CMD_SUCCESS;
+
 	if (CHECK_FLAG(peer->sflags, PEER_STATUS_GROUP))
 		bgp_group_configure_bfd(peer);
 	else
@@ -624,7 +647,7 @@ DEFUN(no_neighbor_bfd_profile, no_neighbor_bfd_profile_cmd,
 }
 #endif /* HAVE_BFDD */
 
-void bgp_bfd_init(struct thread_master *tm)
+void bgp_bfd_init(struct event_loop *tm)
 {
 	/* Initialize BFD client functions */
 	bfd_protocol_integration_init(zclient, tm);
