@@ -2558,7 +2558,7 @@ static int bgp_debug_per_prefix(const struct prefix *p,
 	struct bgp_debug_filter *filter;
 	struct listnode *node, *nnode;
 
-	if (term_bgp_debug_type & BGP_DEBUG_TYPE) {
+	if (CHECK_FLAG(term_bgp_debug_type, BGP_DEBUG_TYPE)) {
 		/* We are debugging all prefixes so return true */
 		if (!per_prefix_list || list_isempty(per_prefix_list))
 			return 1;
@@ -2591,7 +2591,7 @@ static bool bgp_debug_per_peer(char *host, const struct prefix *p,
 	struct bgp_debug_filter *filter;
 	struct listnode *node, *nnode;
 
-	if (term_bgp_debug_type & BGP_DEBUG_TYPE) {
+	if (CHECK_FLAG(term_bgp_debug_type, BGP_DEBUG_TYPE)) {
 		/* We are debugging all peers so return true */
 		if (!per_peer_list || list_isempty(per_peer_list))
 			return true;
