@@ -1,23 +1,10 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IS-IS Rout(e)ing protocol - isis_vty_fabricd.c
  *
  * This file contains the CLI that is specific to OpenFabric
  *
  * Copyright (C) 2018        Christian Franke, for NetDEF, Inc.
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public Licenseas published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <zebra.h>
 
@@ -112,12 +99,13 @@ DEFUN (no_triggered_csnp,
 	return CMD_SUCCESS;
 }
 
-static void lsp_print_flooding(struct vty *vty, struct isis_lsp *lsp)
+static void lsp_print_flooding(struct vty *vty, struct isis_lsp *lsp,
+			       struct isis *isis)
 {
 	char lspid[255];
 	char buf[MONOTIME_STRLEN];
 
-	lspid_print(lsp->hdr.lsp_id, lspid, true, true);
+	lspid_print(lsp->hdr.lsp_id, lspid, sizeof(lspid), true, true, isis);
 	vty_out(vty, "Flooding information for %s\n", lspid);
 
 	if (!lsp->flooding_neighbors[TX_LSP_NORMAL]) {
@@ -136,8 +124,7 @@ static void lsp_print_flooding(struct vty *vty, struct isis_lsp *lsp)
 	vty_out(vty, "%s ago)\n", buf);
 
 	if (lsp->flooding_circuit_scoped) {
-		vty_out(vty, "    Received as circuit-scoped LSP, so not "
-			"flooded.\n");
+		vty_out(vty, "    Received as circuit-scoped LSP, so not flooded.\n");
 		return;
 	}
 
@@ -171,25 +158,29 @@ DEFUN (show_lsp_flooding,
 
 	struct listnode *node;
 	struct isis_area *area;
+	struct isis *isis = NULL;
+
+	isis = isis_lookup_by_vrfid(VRF_DEFAULT);
+
+	if (isis == NULL) {
+		vty_out(vty, "IS-IS Routing Process not enabled\n");
+		return CMD_SUCCESS;
+	}
 
 	for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area)) {
 		struct lspdb_head *head = &area->lspdb[ISIS_LEVEL2 - 1];
 		struct isis_lsp *lsp;
 
-		vty_out(vty, "Area %s:\n", area->area_tag ?
-			area->area_tag : "null");
-
+		vty_out(vty, "Area %s:\n",
+			area->area_tag ? area->area_tag : "null");
 		if (lspid) {
-			lsp = lsp_for_arg(head, lspid);
-
+			lsp = lsp_for_sysid(head, lspid, isis);
 			if (lsp)
-				lsp_print_flooding(vty, lsp);
-
+				lsp_print_flooding(vty, lsp, isis);
 			continue;
 		}
-
 		frr_each (lspdb, head, lsp) {
-			lsp_print_flooding(vty, lsp);
+			lsp_print_flooding(vty, lsp, isis);
 			vty_out(vty, "\n");
 		}
 	}
@@ -223,12 +214,12 @@ DEFUN (ip_router_isis,
 		}
 	}
 
-	area = isis_area_lookup(area_tag);
+	area = isis_area_lookup(area_tag, VRF_DEFAULT);
 	if (!area)
-		area = isis_area_create(area_tag);
+		isis_area_create(area_tag, VRF_DEFAULT_NAME);
 
-	if (!circuit || !circuit->area) {
-		circuit = isis_circuit_create(area, ifp);
+	if (!circuit) {
+		circuit = isis_circuit_new(ifp, area_tag);
 
 		if (circuit->state != C_STATE_CONF
 		    && circuit->state != C_STATE_UP) {
@@ -277,14 +268,14 @@ DEFUN (no_ip_router_isis,
 	const char *af = argv[idx_afi]->arg;
 	const char *area_tag = argv[idx_word]->arg;
 
-	area = isis_area_lookup(area_tag);
+	area = isis_area_lookup(area_tag, VRF_DEFAULT);
 	if (!area) {
 		vty_out(vty, "Can't find ISIS instance %s\n",
 			area_tag);
 		return CMD_ERR_NO_MATCH;
 	}
 
-	circuit = circuit_lookup_by_ifp(ifp, area->circuit_list);
+	circuit = circuit_scan_by_ifp(ifp);
 	if (!circuit) {
 		vty_out(vty, "ISIS is not enabled on circuit %s\n", ifp->name);
 		return CMD_ERR_NO_MATCH;
@@ -297,6 +288,10 @@ DEFUN (no_ip_router_isis,
 		ip = false;
 
 	isis_circuit_af_set(circuit, ip, ipv6);
+
+	if (!ip && !ipv6)
+		isis_circuit_del(circuit);
+
 	return CMD_SUCCESS;
 }
 
@@ -311,13 +306,11 @@ DEFUN (isis_bfd,
 	if (!circuit)
 		return CMD_ERR_NO_MATCH;
 
-	if (circuit->bfd_info
-	    && CHECK_FLAG(circuit->bfd_info->flags, BFD_FLAG_PARAM_CFG)) {
+	if (circuit->bfd_config.enabled)
 		return CMD_SUCCESS;
-	}
 
-	isis_bfd_circuit_param_set(circuit, BFD_DEF_MIN_RX,
-				   BFD_DEF_MIN_TX, BFD_DEF_DETECT_MULT, true);
+	circuit->bfd_config.enabled = true;
+	isis_bfd_circuit_cmd(circuit);
 
 	return CMD_SUCCESS;
 }
@@ -335,11 +328,12 @@ DEFUN (no_isis_bfd,
 	if (!circuit)
 		return CMD_ERR_NO_MATCH;
 
-	if (!circuit->bfd_info)
+	if (!circuit->bfd_config.enabled)
 		return CMD_SUCCESS;
 
-	isis_bfd_circuit_cmd(circuit, ZEBRA_BFD_DEST_DEREGISTER);
-	bfd_info_free(&circuit->bfd_info);
+	circuit->bfd_config.enabled = false;
+	isis_bfd_circuit_cmd(circuit);
+
 	return CMD_SUCCESS;
 }
 
@@ -437,8 +431,7 @@ isis_vty_lsp_gen_interval_set(struct vty *vty, int level, uint16_t interval)
 
 		if (interval >= area->lsp_refresh[lvl - 1]) {
 			vty_out(vty,
-				"LSP gen interval %us must be less than "
-				"the LSP refresh interval %us\n",
+				"LSP gen interval %us must be less than the LSP refresh interval %us\n",
 				interval, area->lsp_refresh[lvl - 1]);
 			return CMD_WARNING_CONFIG_FAILED;
 		}
@@ -488,15 +481,13 @@ isis_vty_lsp_refresh_set(struct vty *vty, int level, uint16_t interval)
 			continue;
 		if (interval <= area->lsp_gen_interval[lvl - 1]) {
 			vty_out(vty,
-				"LSP refresh interval %us must be greater than "
-				"the configured LSP gen interval %us\n",
+				"LSP refresh interval %us must be greater than the configured LSP gen interval %us\n",
 				interval, area->lsp_gen_interval[lvl - 1]);
 			return CMD_WARNING_CONFIG_FAILED;
 		}
 		if (interval > (area->max_lsp_lifetime[lvl - 1] - 300)) {
 			vty_out(vty,
-				"LSP refresh interval %us must be less than "
-				"the configured LSP lifetime %us less 300\n",
+				"LSP refresh interval %us must be less than the configured LSP lifetime %us less 300\n",
 				interval, area->max_lsp_lifetime[lvl - 1]);
 			return CMD_WARNING_CONFIG_FAILED;
 		}
@@ -546,20 +537,17 @@ isis_vty_max_lsp_lifetime_set(struct vty *vty, int level, uint16_t interval)
 
 		if (refresh_interval < area->lsp_refresh[lvl - 1]) {
 			vty_out(vty,
-				"Level %d Max LSP lifetime %us must be 300s greater than "
-				"the configured LSP refresh interval %us\n",
+				"Level %d Max LSP lifetime %us must be 300s greater than the configured LSP refresh interval %us\n",
 				lvl, interval, area->lsp_refresh[lvl - 1]);
 			vty_out(vty,
-				"Automatically reducing level %d LSP refresh interval "
-				"to %us\n",
+				"Automatically reducing level %d LSP refresh interval to %us\n",
 				lvl, refresh_interval);
 			set_refresh_interval[lvl - 1] = 1;
 
 			if (refresh_interval
 			    <= area->lsp_gen_interval[lvl - 1]) {
 				vty_out(vty,
-					"LSP refresh interval %us must be greater than "
-					"the configured LSP gen interval %us\n",
+					"LSP refresh interval %us must be greater than the configured LSP gen interval %us\n",
 					refresh_interval,
 					area->lsp_gen_interval[lvl - 1]);
 				return CMD_WARNING_CONFIG_FAILED;
@@ -845,8 +833,7 @@ DEFUN (isis_metric,
 	if (circuit->area && circuit->area->oldmetric == 1
 	    && met > MAX_NARROW_LINK_METRIC) {
 		vty_out(vty,
-			"Invalid metric %d - should be <0-63> "
-			"when narrow metric type enabled\n",
+			"Invalid metric %d - should be <0-63> when narrow metric type enabled\n",
 			met);
 		return CMD_WARNING_CONFIG_FAILED;
 	}
@@ -855,8 +842,7 @@ DEFUN (isis_metric,
 	if (circuit->area && circuit->area->newmetric == 1
 	    && met > MAX_WIDE_LINK_METRIC) {
 		vty_out(vty,
-			"Invalid metric %d - should be <0-16777215> "
-			"when wide metric type enabled\n",
+			"Invalid metric %d - should be <0-16777215> when wide metric type enabled\n",
 			met);
 		return CMD_WARNING_CONFIG_FAILED;
 	}
