@@ -505,21 +505,12 @@ nbr_start_idtimer(struct nbr *nbr)
 {
 	int	secs;
 
-	secs = INIT_DELAY_TMR;
-	switch(nbr->idtimer_cnt) {
-	default:
+	if (nbr->idtimer_cnt > 2) {
 		/* do not further increase the counter */
 		secs = MAX_DELAY_TMR;
-		break;
-	case 2:
-		secs *= 2;
-		/* FALLTHROUGH */
-	case 1:
-		secs *= 2;
-		/* FALLTHROUGH */
-	case 0:
+	} else {
+		secs = INIT_DELAY_TMR * (1 << nbr->idtimer_cnt);
 		nbr->idtimer_cnt++;
-		break;
 	}
 
 	EVENT_OFF(nbr->initdelay_timer);
@@ -690,6 +681,18 @@ nbr_gtsm_setup(int fd, int af, struct nbr_params *nbrp)
 	if (nbrp && CHECK_FLAG(nbrp->flags, F_NBRP_GTSM_HOPS))
 		ttl = 256 - nbrp->gtsm_hops;
 
+	/*
+	 * In linux networking stack, the received mpls packets
+	 * will be processed by the host twice, one as mpls packet,
+	 * the other as ip packet, so its ttl will be decreased 1.
+	 * This behavior is based on the new kernel (5.10 and 6.1),
+	 * and older versions may behave differently.
+	 *
+	 * Here, decrease 1 for IP_MINTTL if GTSM is enabled.
+	 * And this workaround makes the GTSM mechanism a bit deviation.
+	 */
+	ttl -= 1;
+
 	switch (af) {
 	case AF_INET:
 		if (sock_set_ipv4_minttl(fd, ttl) == -1)
