@@ -32,6 +32,8 @@ enum bfd_session_event {
 	BSE_UNINSTALL,
 	/** Install the BFD session configuration. */
 	BSE_INSTALL,
+	/** We should install but it couldn't because of a error talking to zebra */
+	BSE_VALID_FOR_INSTALL,
 };
 
 /**
@@ -527,6 +529,10 @@ static void _bfd_sess_send(struct event *t)
 			vrf_id_to_name(bsp->args.vrf_id), bsp->args.vrf_id,
 			bsp->lastev == BSE_INSTALL ? "installed"
 						   : "uninstalled");
+
+		bsp->installed = false;
+		if (bsp->lastev == BSE_INSTALL)
+			bsp->lastev = BSE_VALID_FOR_INSTALL;
 	}
 }
 
@@ -883,7 +889,7 @@ int zclient_bfd_session_replay(ZAPI_CALLBACK_ARGS)
 	/* Replay all activated peers. */
 	TAILQ_FOREACH (bsp, &bsglobal.bsplist, entry) {
 		/* Skip not installed sessions. */
-		if (!bsp->installed)
+		if (!bsp->installed && bsp->lastev != BSE_VALID_FOR_INSTALL)
 			continue;
 
 		/* We are reconnecting, so we must send installation. */
@@ -1282,7 +1288,6 @@ static bool bfd_source_cache_update(struct bfd_source_cache *source,
 		const struct zapi_nexthop *nh = &route->nexthops[nh_index];
 		const struct interface *interface;
 		const struct connected *connected;
-		const struct listnode *node;
 
 		interface = if_lookup_by_index(nh->ifindex, nh->vrf_id);
 		if (interface == NULL) {
@@ -1291,8 +1296,7 @@ static bool bfd_source_cache_update(struct bfd_source_cache *source,
 			continue;
 		}
 
-		for (ALL_LIST_ELEMENTS_RO(interface->connected, node,
-					  connected)) {
+		frr_each (if_connected_const, interface->connected, connected) {
 			if (source->address.family !=
 			    connected->address->family)
 				continue;
@@ -1336,3 +1340,9 @@ int bfd_nht_update(const struct prefix *match, const struct zapi_route *route)
 
 	return 0;
 }
+
+bool bfd_session_is_down(const struct bfd_session_params *session)
+{
+	return session->bss.state == BSS_DOWN ||
+	       session->bss.state == BSS_ADMIN_DOWN;
+}
