@@ -358,6 +358,9 @@ int frrscript_load(struct frrscript *fs, const char *function_name,
 	/* Set up the Lua script */
 	lua_State *L = luaL_newstate();
 
+	/* Load basic built-in Lua functions, e.g. ipairs, string, etc. */
+	luaL_openlibs(L);
+
 	frrlua_export_logging(L);
 
 	char script_name[MAXPATHLEN];
