@@ -369,6 +369,11 @@ struct nb_callbacks {
 	 */
 	int (*destroy)(struct nb_cb_destroy_args *args);
 
+	/*
+	 * Flags to control the how northbound callbacks are invoked.
+	 */
+	uint flags;
+
 	/*
 	 * Configuration callback.
 	 *
@@ -587,6 +592,12 @@ struct nb_callbacks {
 	void (*cli_show_end)(struct vty *vty, const struct lyd_node *dnode);
 };
 
+/*
+ * Flag indicating the northbound should recurse destroy the children of this
+ * node when it is destroyed.
+ */
+#define F_NB_CB_DESTROY_RECURSE 0x01
+
 struct nb_dependency_callbacks {
 	void (*get_dependant_xpath)(const struct lyd_node *dnode, char *xpath);
 	void (*get_dependency_xpath)(const struct lyd_node *dnode, char *xpath);
@@ -678,6 +689,7 @@ enum nb_error {
 	NB_ERR,
 	NB_ERR_NO_CHANGES,
 	NB_ERR_NOT_FOUND,
+	NB_ERR_EXISTS,
 	NB_ERR_LOCKED,
 	NB_ERR_VALIDATION,
 	NB_ERR_RESOURCE,
@@ -799,8 +811,6 @@ DECLARE_HOOK(nb_notification_send, (const char *xpath, struct list *arguments),
 	     (xpath, arguments));
 DECLARE_HOOK(nb_notification_tree_send,
 	     (const char *xpath, const struct lyd_node *tree), (xpath, tree));
-DECLARE_HOOK(nb_client_debug_config_write, (struct vty *vty), (vty));
-DECLARE_HOOK(nb_client_debug_set_all, (uint32_t flags, bool set), (flags, set));
 
 /* Northbound debugging records */
 extern struct debug nb_dbg_cbs_config;
@@ -1017,6 +1027,9 @@ extern int nb_candidate_edit(struct nb_config *candidate,
  * data
  *    New data tree for the node.
  *
+ * created
+ *    OUT param set accordingly if a node was created or just updated
+ *
  * xpath_created
  *    XPath of the created node if operation is "create".
  *
@@ -1031,9 +1044,9 @@ extern int nb_candidate_edit(struct nb_config *candidate,
  *    - NB_ERR for other errors.
  */
 extern int nb_candidate_edit_tree(struct nb_config *candidate,
-				  enum nb_operation operation,
-				  LYD_FORMAT format, const char *xpath,
-				  const char *data, char *xpath_created,
+				  enum nb_operation operation, LYD_FORMAT format,
+				  const char *xpath, const char *data,
+				  bool *created, char *xpath_created,
 				  char *errmsg, size_t errmsg_len);
 
 /*
@@ -1701,10 +1714,12 @@ void nb_validate_callbacks(void);
  *
  * db_enabled
  *    Set this to record the transactions in the transaction log.
+ *
+ * load_library
+ *    Set this to have libyang to load/implement the ietf-yang-library.
  */
-extern void nb_init(struct event_loop *tm,
-		    const struct frr_yang_module_info *const modules[],
-		    size_t nmodules, bool db_enabled);
+extern void nb_init(struct event_loop *tm, const struct frr_yang_module_info *const modules[],
+		    size_t nmodules, bool db_enabled, bool load_library);
 
 /*
  * Finish the northbound layer gracefully. Should be called only when the daemon
@@ -1714,6 +1729,7 @@ extern void nb_terminate(void);
 
 extern void nb_oper_init(struct event_loop *loop);
 extern void nb_oper_terminate(void);
+extern bool nb_oper_is_yang_lib_query(const char *xpath);
 
 #ifdef __cplusplus
 }
