@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2003 Yasuhiro Ohara
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -25,18 +10,21 @@
 #include "sockunion.h"
 #include "sockopt.h"
 #include "privs.h"
+#include "lib_errors.h"
+#include "vrf.h"
 
 #include "libospf.h"
 #include "ospf6_proto.h"
+#include "ospf6_top.h"
 #include "ospf6_network.h"
 #include "ospf6d.h"
+#include "ospf6_message.h"
 
-int ospf6_sock;
 struct in6_addr allspfrouters6;
 struct in6_addr alldrouters6;
 
 /* setsockopt MulticastLoop to off */
-static void ospf6_reset_mcastloop(void)
+static void ospf6_reset_mcastloop(int ospf6_sock)
 {
 	unsigned int off = 0;
 	if (setsockopt(ospf6_sock, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, &off,
@@ -46,47 +34,47 @@ static void ospf6_reset_mcastloop(void)
 			  safe_strerror(errno));
 }
 
-static void ospf6_set_pktinfo(void)
+static void ospf6_set_pktinfo(int ospf6_sock)
 {
 	setsockopt_ipv6_pktinfo(ospf6_sock, 1);
 }
 
-static void ospf6_set_transport_class(void)
+static void ospf6_set_transport_class(int ospf6_sock)
 {
 #ifdef IPTOS_PREC_INTERNETCONTROL
 	setsockopt_ipv6_tclass(ospf6_sock, IPTOS_PREC_INTERNETCONTROL);
 #endif
 }
 
-static void ospf6_set_checksum(void)
+void ospf6_serv_close(int *ospf6_sock)
 {
-	int offset = 12;
-#ifndef DISABLE_IPV6_CHECKSUM
-	if (setsockopt(ospf6_sock, IPPROTO_IPV6, IPV6_CHECKSUM, &offset,
-		       sizeof(offset))
-	    < 0)
-		zlog_warn("Network: set IPV6_CHECKSUM failed: %s",
-			  safe_strerror(errno));
-#else
-	zlog_warn("Network: Don't set IPV6_CHECKSUM");
-#endif /* DISABLE_IPV6_CHECKSUM */
+	if (*ospf6_sock != -1) {
+		close(*ospf6_sock);
+		*ospf6_sock = -1;
+		return;
+	}
 }
 
 /* Make ospf6d's server socket. */
-int ospf6_serv_sock(void)
+int ospf6_serv_sock(struct ospf6 *ospf6)
 {
-	if (ospf6d_privs.change(ZPRIVS_RAISE))
-		zlog_err("ospf6_serv_sock: could not raise privs");
+	int ospf6_sock;
 
-	ospf6_sock = socket(AF_INET6, SOCK_RAW, IPPROTO_OSPFIGP);
-	if (ospf6_sock < 0) {
-		zlog_warn("Network: can't create OSPF6 socket.");
-		if (ospf6d_privs.change(ZPRIVS_LOWER))
-			zlog_err("ospf_sock_init: could not lower privs");
+	if (ospf6->fd != -1)
 		return -1;
+
+	if (ospf6->vrf_id == VRF_UNKNOWN)
+		return -1;
+
+	frr_with_privs(&ospf6d_privs) {
+
+		ospf6_sock = vrf_socket(AF_INET6, SOCK_RAW, IPPROTO_OSPFIGP,
+					ospf6->vrf_id, ospf6->name);
+		if (ospf6_sock < 0) {
+			zlog_warn("Network: can't create OSPF6 socket.");
+			return -1;
+		}
 	}
-	if (ospf6d_privs.change(ZPRIVS_LOWER))
-		zlog_err("ospf_sock_init: could not lower privs");
 
 /* set socket options */
 #if 1
@@ -94,11 +82,11 @@ int ospf6_serv_sock(void)
 #else
 	ospf6_set_reuseaddr();
 #endif /*1*/
-	ospf6_reset_mcastloop();
-	ospf6_set_pktinfo();
-	ospf6_set_transport_class();
-	ospf6_set_checksum();
+	ospf6_reset_mcastloop(ospf6_sock);
+	ospf6_set_pktinfo(ospf6_sock);
+	ospf6_set_transport_class(ospf6_sock);
 
+	ospf6->fd = ospf6_sock;
 	/* setup global in6_addr, allspf6 and alldr6 for later use */
 	inet_pton(AF_INET6, ALLSPFROUTERS6, &allspfrouters6);
 	inet_pton(AF_INET6, ALLDROUTERS6, &alldrouters6);
@@ -107,26 +95,30 @@ int ospf6_serv_sock(void)
 }
 
 /* ospf6 set socket option */
-int ospf6_sso(ifindex_t ifindex, struct in6_addr *group, int option)
+int ospf6_sso(ifindex_t ifindex, struct in6_addr *group, int option, int sockfd)
 {
 	struct ipv6_mreq mreq6;
 	int ret;
 	int bufsize = (8 * 1024 * 1024);
 
+	if (sockfd == -1)
+		return -1;
+
 	assert(ifindex);
 	mreq6.ipv6mr_interface = ifindex;
 	memcpy(&mreq6.ipv6mr_multiaddr, group, sizeof(struct in6_addr));
 
-	ret = setsockopt(ospf6_sock, IPPROTO_IPV6, option, &mreq6,
-			 sizeof(mreq6));
+	ret = setsockopt(sockfd, IPPROTO_IPV6, option, &mreq6, sizeof(mreq6));
 	if (ret < 0) {
-		zlog_err("Network: setsockopt (%d) on ifindex %d failed: %s",
-			 option, ifindex, safe_strerror(errno));
+		flog_err_sys(
+			EC_LIB_SOCKET,
+			"Network: setsockopt (%d) on ifindex %d failed: %s",
+			option, ifindex, safe_strerror(errno));
 		return ret;
 	}
 
-	setsockopt_so_sendbuf(ospf6_sock, bufsize);
-	setsockopt_so_recvbuf(ospf6_sock, bufsize);
+	setsockopt_so_sendbuf(sockfd, bufsize);
+	setsockopt_so_recvbuf(sockfd, bufsize);
 
 	return 0;
 }
@@ -149,7 +141,7 @@ static int iov_totallen(struct iovec *iov)
 }
 
 int ospf6_sendmsg(struct in6_addr *src, struct in6_addr *dst,
-		  ifindex_t *ifindex, struct iovec *message)
+		  ifindex_t ifindex, struct iovec *message, int ospf6_sock)
 {
 	int retval;
 	struct msghdr smsghdr;
@@ -162,14 +154,14 @@ int ospf6_sendmsg(struct in6_addr *src, struct in6_addr *dst,
 	struct sockaddr_in6 dst_sin6;
 
 	assert(dst);
-	assert(*ifindex);
 
+	memset(&cmsgbuf, 0, sizeof(cmsgbuf));
 	scmsgp = (struct cmsghdr *)&cmsgbuf;
 	pktinfo = (struct in6_pktinfo *)(CMSG_DATA(scmsgp));
-	memset(&dst_sin6, 0, sizeof(struct sockaddr_in6));
+	memset(&dst_sin6, 0, sizeof(dst_sin6));
 
 	/* source address */
-	pktinfo->ipi6_ifindex = *ifindex;
+	pktinfo->ipi6_ifindex = ifindex;
 	if (src)
 		memcpy(&pktinfo->ipi6_addr, src, sizeof(struct in6_addr));
 	else
@@ -181,7 +173,7 @@ int ospf6_sendmsg(struct in6_addr *src, struct in6_addr *dst,
 	dst_sin6.sin6_len = sizeof(struct sockaddr_in6);
 #endif /*SIN6_LEN*/
 	memcpy(&dst_sin6.sin6_addr, dst, sizeof(struct in6_addr));
-	dst_sin6.sin6_scope_id = *ifindex;
+	dst_sin6.sin6_scope_id = ifindex;
 
 	/* send control msg */
 	scmsgp->cmsg_level = IPPROTO_IPV6;
@@ -200,14 +192,15 @@ int ospf6_sendmsg(struct in6_addr *src, struct in6_addr *dst,
 
 	retval = sendmsg(ospf6_sock, &smsghdr, 0);
 	if (retval != iov_totallen(message))
-		zlog_warn("sendmsg failed: ifindex: %d: %s (%d)", *ifindex,
+		zlog_warn("sendmsg failed: source: %pI6 Dest: %pI6 ifindex: %d: %s (%d)",
+			  src, dst, ifindex,
 			  safe_strerror(errno), errno);
 
 	return retval;
 }
 
 int ospf6_recvmsg(struct in6_addr *src, struct in6_addr *dst,
-		  ifindex_t *ifindex, struct iovec *message)
+		  ifindex_t *ifindex, struct iovec *message, int ospf6_sock)
 {
 	int retval;
 	struct msghdr rmsghdr;
@@ -218,7 +211,7 @@ int ospf6_recvmsg(struct in6_addr *src, struct in6_addr *dst,
 
 	rcmsgp = (struct cmsghdr *)cmsgbuf;
 	pktinfo = (struct in6_pktinfo *)(CMSG_DATA(rcmsgp));
-	memset(&src_sin6, 0, sizeof(struct sockaddr_in6));
+	memset(&src_sin6, 0, sizeof(src_sin6));
 
 	/* receive control msg */
 	rcmsgp->cmsg_level = IPPROTO_IPV6;
@@ -235,10 +228,13 @@ int ospf6_recvmsg(struct in6_addr *src, struct in6_addr *dst,
 	rmsghdr.msg_control = (caddr_t)cmsgbuf;
 	rmsghdr.msg_controllen = sizeof(cmsgbuf);
 
-	retval = recvmsg(ospf6_sock, &rmsghdr, 0);
-	if (retval < 0)
-		zlog_warn("recvmsg failed: %s", safe_strerror(errno));
-	else if (retval == iov_totallen(message))
+	retval = recvmsg(ospf6_sock, &rmsghdr, MSG_DONTWAIT);
+	if (retval < 0) {
+		if (errno != EAGAIN && errno != EWOULDBLOCK)
+			zlog_warn("stream_recvmsg failed: %s",
+				  safe_strerror(errno));
+		return retval;
+	} else if (retval == iov_totallen(message))
 		zlog_warn("recvmsg read full buffer size: %d", retval);
 
 	/* source address */
