@@ -363,7 +363,7 @@ void ospf6_route_zebra_copy_nexthops(struct ospf6_route *route,
 
 			case NEXTHOP_TYPE_IPV6_IFINDEX:
 				nexthops[i].ifindex = nh->ifindex;
-				/* FALLTHROUGH */
+				fallthrough;
 			case NEXTHOP_TYPE_IPV6:
 				nexthops[i].gate.ipv6 = nh->address;
 				break;
@@ -540,6 +540,10 @@ int ospf6_route_cmp(struct ospf6_route *ra, struct ospf6_route *rb)
 	if (ra->path.area_id != rb->path.area_id)
 		return (ntohl(ra->path.area_id) - ntohl(rb->path.area_id));
 
+	if ((ra->prefix_options & OSPF6_PREFIX_OPTION_LA)
+	    != (rb->prefix_options & OSPF6_PREFIX_OPTION_LA))
+		return ra->prefix_options & OSPF6_PREFIX_OPTION_LA ? -1 : 1;
+
 	return 0;
 }
 
