@@ -62,6 +62,11 @@ DEFINE_MTYPE_STATIC(OSPFD, APISERVER_MSGFILTER, "API Server Message Filter");
 /* List of all active connections. */
 struct list *apiserver_list;
 
+/* Indicates that API the server socket local addresss has been
+ * specified.
+ */
+struct in_addr ospf_apiserver_addr;
+
 /* -----------------------------------------------------------
  * Functions to lookup interfaces
  * -----------------------------------------------------------
@@ -109,7 +114,21 @@ struct ospf_interface *ospf_apiserver_if_lookup_by_ifp(struct interface *ifp)
 
 unsigned short ospf_apiserver_getport(void)
 {
-	struct servent *sp = getservbyname("ospfapi", "tcp");
+	struct servent *sp = NULL;
+	char sbuf[16];
+
+	/*
+	 * Allow the OSPF API server port to be specified per-instance by
+	 * including the instance ID in the /etc/services name. Use the
+	 * prior name if no per-instance service is specified.
+	 */
+	if (ospf_instance) {
+		snprintfrr(sbuf, sizeof(sbuf), "ospfapi-%d", ospf_instance);
+		sp = getservbyname(sbuf, "tcp");
+	}
+
+	if (!sp)
+		sp = getservbyname("ospfapi", "tcp");
 
 	return sp ? ntohs(sp->s_port) : OSPF_API_SYNC_PORT;
 }
@@ -557,8 +576,10 @@ int ospf_apiserver_serv_sock_family(unsigned short port, int family)
 	sockopt_reuseaddr(accept_sock);
 	sockopt_reuseport(accept_sock);
 
-	/* Bind socket to address and given port. */
-	rc = sockunion_bind(accept_sock, &su, port, NULL);
+	/* Bind socket to optional lcoal address and port. */
+	if (ospf_apiserver_addr.s_addr)
+		sockunion2ip(&su) = ospf_apiserver_addr.s_addr;
+	rc = sockunion_bind(accept_sock, &su, port, &su);
 	if (rc < 0) {
 		close(accept_sock); /* Close socket */
 		return rc;
@@ -984,6 +1005,8 @@ void ospf_apiserver_notify_ready_type9(struct ospf_apiserver *apiserv)
 	struct registered_opaque_type *r;
 
 	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+	if (!ospf)
+		goto out;
 
 	for (ALL_LIST_ELEMENTS(ospf->oiflist, node, nnode, oi)) {
 		/* Check if this interface is indeed ready for type 9 */
@@ -1032,6 +1055,8 @@ void ospf_apiserver_notify_ready_type10(struct ospf_apiserver *apiserv)
 	struct ospf_area *area;
 
 	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+	if (!ospf)
+		goto out;
 
 	for (ALL_LIST_ELEMENTS(ospf->areas, node, nnode, area)) {
 		struct registered_opaque_type *r;
@@ -1079,6 +1104,8 @@ void ospf_apiserver_notify_ready_type11(struct ospf_apiserver *apiserv)
 	struct registered_opaque_type *r;
 
 	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+	if (!ospf)
+		goto out;
 
 	/* Can type 11 be originated? */
 	if (!ospf_apiserver_is_ready_type11(ospf))
@@ -1255,10 +1282,13 @@ int ospf_apiserver_handle_sync_lsdb(struct ospf_apiserver *apiserv,
 	struct ospf *ospf;
 	struct ospf_area *area;
 
-	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
-
 	/* Get request sequence number */
 	seqnum = msg_get_seq(msg);
+
+	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+	if (!ospf)
+		goto out;
+
 	/* Set sync msg. */
 	smsg = (struct msg_sync_lsdb *)STREAM_DATA(msg->s);
 
@@ -1336,6 +1366,7 @@ int ospf_apiserver_handle_sync_lsdb(struct ospf_apiserver *apiserv,
 							seqnum);
 	}
 
+out:
 	/* Send a reply back to client with return code */
 	rc = ospf_apiserver_send_reply(apiserv, seqnum, rc);
 	return rc;
@@ -1350,14 +1381,19 @@ int ospf_apiserver_handle_sync_lsdb(struct ospf_apiserver *apiserv,
 int ospf_apiserver_handle_sync_reachable(struct ospf_apiserver *apiserv,
 					 struct msg *msg)
 {
-	struct ospf *ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
-	struct route_table *rt = ospf->all_rtrs;
+	int _rc, rc = 0;
 	uint32_t seqnum = msg_get_seq(msg);
+	struct ospf *ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+
+	if (!ospf)
+		goto out;
+
+	struct route_table *rt = ospf->all_rtrs;
 	struct in_addr *a, *abuf;
 	struct msg_reachable_change *areach;
 	struct msg *amsg;
 	uint mcount, count;
-	int _rc, rc = 0;
+
 
 	if (!rt)
 		goto out;
@@ -1395,13 +1431,17 @@ out:
 int ospf_apiserver_handle_sync_ism(struct ospf_apiserver *apiserv,
 				   struct msg *msg)
 {
+	uint32_t seqnum = msg_get_seq(msg);
+	int _rc, rc = 0;
 	struct ospf *ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+
+	if (!ospf)
+		goto out;
+
 	struct listnode *anode, *inode;
 	struct ospf_area *area;
 	struct ospf_interface *oi;
 	struct msg *m;
-	uint32_t seqnum = msg_get_seq(msg);
-	int _rc, rc = 0;
 
 	/* walk all areas */
 	for (ALL_LIST_ELEMENTS_RO(ospf->areas, anode, area)) {
@@ -1417,6 +1457,7 @@ int ospf_apiserver_handle_sync_ism(struct ospf_apiserver *apiserv,
 		if (rc)
 			break;
 	}
+out:
 	/* Send a reply back to client with return code */
 	_rc = ospf_apiserver_send_reply(apiserv, seqnum, rc);
 	return rc ? rc : _rc;
@@ -1426,15 +1467,19 @@ int ospf_apiserver_handle_sync_ism(struct ospf_apiserver *apiserv,
 int ospf_apiserver_handle_sync_nsm(struct ospf_apiserver *apiserv,
 				   struct msg *msg)
 {
+	uint32_t seqnum = msg_get_seq(msg);
+	int _rc, rc = 0;
 	struct ospf *ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+
+	if (!ospf)
+		goto out;
+
 	struct listnode *anode, *inode;
 	struct ospf_area *area;
 	struct ospf_interface *oi;
 	struct ospf_neighbor *nbr;
 	struct route_node *rn;
 	struct msg *m;
-	uint32_t seqnum = msg_get_seq(msg);
-	int _rc, rc = 0;
 
 	/* walk all areas */
 	for (ALL_LIST_ELEMENTS_RO(ospf->areas, anode, area)) {
@@ -1460,6 +1505,7 @@ int ospf_apiserver_handle_sync_nsm(struct ospf_apiserver *apiserv,
 		if (rc)
 			break;
 	}
+out:
 	/* Send a reply back to client with return code */
 	_rc = ospf_apiserver_send_reply(apiserv, seqnum, rc);
 	return rc ? rc : _rc;
@@ -1469,15 +1515,20 @@ int ospf_apiserver_handle_sync_nsm(struct ospf_apiserver *apiserv,
 int ospf_apiserver_handle_sync_router_id(struct ospf_apiserver *apiserv,
 					 struct msg *msg)
 {
-	struct ospf *ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 	uint32_t seqnum = msg_get_seq(msg);
 	struct msg *m;
 	int _rc, rc = 0;
+	struct ospf *ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
+
+	if (!ospf)
+		goto out;
+
 
 	m = new_msg_router_id_change(seqnum, ospf->router_id);
 	rc = ospf_apiserver_send_msg(apiserv, m);
 	msg_free(m);
 
+out:
 	/* Send a reply back to client with return code */
 	_rc = ospf_apiserver_send_reply(apiserv, seqnum, rc);
 	return rc ? rc : _rc;
@@ -1512,7 +1563,8 @@ struct ospf_lsa *ospf_apiserver_opaque_lsa_new(struct ospf_area *area,
 	else
 		ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
-	assert(ospf);
+	if (!ospf)
+		return NULL;
 
 	/* Create a stream for internal opaque LSA */
 	if ((s = stream_new(OSPF_MAX_LSA_SIZE)) == NULL) {
@@ -1600,6 +1652,9 @@ int ospf_apiserver_handle_originate_request(struct ospf_apiserver *apiserv,
 
 	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
 
+	if (!ospf)
+		goto out;
+
 	/* Extract opaque LSA data from message */
 	omsg = (struct msg_originate_request *)STREAM_DATA(msg->s);
 	data = &omsg->data;
@@ -1740,7 +1795,8 @@ void ospf_apiserver_flood_opaque_lsa(struct ospf_lsa *lsa)
 		struct ospf *ospf;
 
 		ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
-		assert(ospf);
+		if (!ospf)
+			return;
 
 		/* Increment counters? XXX */
 
@@ -1756,7 +1812,8 @@ int ospf_apiserver_originate1(struct ospf_lsa *lsa, struct ospf_lsa *old)
 	struct ospf *ospf;
 
 	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
-	assert(ospf);
+	if (!ospf)
+		return -1;
 
 	if (old) {
 		/*
@@ -1855,7 +1912,8 @@ struct ospf_lsa *ospf_apiserver_lsa_refresher(struct ospf_lsa *lsa)
 	assert(lsa);
 
 	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
-	assert(ospf);
+	if (!ospf)
+		return NULL;
 
 	if (IS_DEBUG_OSPF(lsa, LSA_GENERATE)) {
 		zlog_debug("LSA[Type%d:%pI4]: OSPF API Server LSA Refresher",
@@ -1944,7 +2002,8 @@ int ospf_apiserver_handle_delete_request(struct ospf_apiserver *apiserv,
 	struct ospf *ospf;
 
 	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
-	assert(ospf);
+	if (!ospf)
+		goto out;
 
 	/* Extract opaque LSA from message */
 	dmsg = (struct msg_delete_request *)STREAM_DATA(msg->s);
@@ -2068,7 +2127,8 @@ void ospf_apiserver_flush_opaque_lsa(struct ospf_apiserver *apiserv,
 	struct ospf_area *area;
 
 	ospf = ospf_lookup_by_vrf_id(VRF_DEFAULT);
-	assert(ospf);
+	if (!ospf)
+		return;
 
 	/* Set parameter struct. */
 	param.apiserv = apiserv;
@@ -2092,7 +2152,7 @@ void ospf_apiserver_flush_opaque_lsa(struct ospf_apiserver *apiserv,
 					lsa, (void *)&param, 0);
 		break;
 	case OSPF_OPAQUE_AS_LSA:
-		LSDB_LOOP (OPAQUE_LINK_LSDB(ospf), rn, lsa)
+		LSDB_LOOP (OPAQUE_AS_LSDB(ospf), rn, lsa)
 			apiserver_flush_opaque_type_callback(lsa,
 							     (void *)&param, 0);
 		break;
