@@ -83,6 +83,12 @@ static void pim_autorp_free(struct pim_autorp *autorp)
 	pim_autorp_rp_fini(&(autorp->candidate_rp_list));
 }
 
+static bool pim_autorp_should_close(struct pim_autorp *autorp)
+{
+	/* If discovery is active, then we need the socket open */
+	return !autorp->do_discovery;
+}
+
 static bool pim_autorp_join_groups(struct interface *ifp)
 {
 	struct pim_interface *pim_ifp;
@@ -489,6 +495,11 @@ err:
 static bool pim_autorp_socket_enable(struct pim_autorp *autorp)
 {
 	int fd;
+	struct interface *ifp;
+
+	/* Return early if socket is already enabled */
+	if (autorp->sock != -1)
+		return true;
 
 	frr_with_privs (&pimd_privs) {
 		fd = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
@@ -498,16 +509,30 @@ static bool pim_autorp_socket_enable(struct pim_autorp *autorp)
 			return false;
 		}
 
+		if (vrf_bind(autorp->pim->vrf->vrf_id, fd, NULL)) {
+			zlog_warn("Could not bind autorp socket to vrf fd=%d: vrf_id=%d: errno=%d: %s",
+				  fd, autorp->pim->vrf->vrf_id, errno, safe_strerror(errno));
+			close(fd);
+			return false;
+		}
+
 		autorp->sock = fd;
+
 		if (!pim_autorp_setup(autorp)) {
-			zlog_warn("Could not setup autorp socket fd=%d: errno=%d: %s",
-				  fd, errno, safe_strerror(errno));
+			zlog_warn("Could not setup autorp socket fd=%d: errno=%d: %s", fd, errno,
+				  safe_strerror(errno));
+
 			close(fd);
 			autorp->sock = -1;
 			return false;
 		}
 	}
 
+	/* Join autorp groups on all pim enabled interfaces in the VRF */
+	FOR_ALL_INTERFACES (autorp->pim->vrf, ifp) {
+		pim_autorp_add_ifp(ifp);
+	}
+
 	if (PIM_DEBUG_AUTORP)
 		zlog_debug("%s: AutoRP socket enabled", __func__);
 
@@ -516,6 +541,11 @@ static bool pim_autorp_socket_enable(struct pim_autorp *autorp)
 
 static bool pim_autorp_socket_disable(struct pim_autorp *autorp)
 {
+	/* Return early if socket is already disabled */
+	if (autorp->sock == -1)
+		return true;
+
+	/* No need to leave the autorp groups explicitly, they are left when the socket is closed */
 	if (close(autorp->sock)) {
 		zlog_warn("Failure closing autorp socket: fd=%d errno=%d: %s",
 			  autorp->sock, errno, safe_strerror(errno));
@@ -912,10 +942,10 @@ void pim_autorp_add_ifp(struct interface *ifp)
 {
 	/* Add a new interface for autorp
 	 *   When autorp is enabled, we must join the autorp groups on all
-	 *   pim/multicast interfaces. When autorp first starts, if finds all
-	 *   current multicast interfaces and joins on them. If a new interface
-	 *   comes up or is configured for multicast after autorp is running, then
-	 *   this method will add it for autorp->
+	 *   pim/multicast interfaces. When autorp becomes enabled, it finds all
+	 *   current pim enabled interfaces and joins the autorp groups on them.
+	 *   Any new interfaces added after autorp is enabled will use this function
+	 *   to join the autorp groups
 	 * This is called even when adding a new pim interface that is not yet
 	 * active, so make sure the check, it'll call in again once the interface is up.
 	 */
@@ -926,7 +956,8 @@ void pim_autorp_add_ifp(struct interface *ifp)
 	if (CHECK_FLAG(ifp->status, ZEBRA_INTERFACE_ACTIVE) && pim_ifp &&
 	    pim_ifp->pim_enable) {
 		pim = pim_ifp->pim;
-		if (pim && pim->autorp && pim->autorp->do_discovery) {
+		if (pim && pim->autorp &&
+		    (pim->autorp->do_discovery)) {
 			if (PIM_DEBUG_AUTORP)
 				zlog_debug("%s: Adding interface %s to AutoRP, joining AutoRP groups",
 					   __func__, ifp->name);
@@ -964,38 +995,41 @@ void pim_autorp_rm_ifp(struct interface *ifp)
 
 void pim_autorp_start_discovery(struct pim_instance *pim)
 {
-	struct interface *ifp;
 	struct pim_autorp *autorp = pim->autorp;
 
-	if (!autorp->do_discovery) {
-		autorp->do_discovery = true;
-		autorp_read_on(autorp);
+	if (autorp->do_discovery)
+		return;
 
-		FOR_ALL_INTERFACES (autorp->pim->vrf, ifp) {
-			pim_autorp_add_ifp(ifp);
-		}
+	autorp->do_discovery = true;
 
-		if (PIM_DEBUG_AUTORP)
-			zlog_debug("%s: AutoRP Discovery started", __func__);
+	/* Make sure the socket is open and ready */
+	if (!pim_autorp_socket_enable(autorp)) {
+		zlog_err("%s: AutoRP failed to open socket", __func__);
+		return;
 	}
+
+	autorp_read_on(autorp);
+
+	if (PIM_DEBUG_AUTORP)
+		zlog_debug("%s: AutoRP Discovery started", __func__);
 }
 
 void pim_autorp_stop_discovery(struct pim_instance *pim)
 {
-	struct interface *ifp;
 	struct pim_autorp *autorp = pim->autorp;
 
-	if (autorp->do_discovery) {
-		FOR_ALL_INTERFACES (autorp->pim->vrf, ifp) {
-			pim_autorp_rm_ifp(ifp);
-		}
+	if (!autorp->do_discovery)
+		return;
 
-		autorp->do_discovery = false;
-		autorp_read_off(autorp);
+	autorp->do_discovery = false;
+	autorp_read_off(autorp);
 
-		if (PIM_DEBUG_AUTORP)
-			zlog_debug("%s: AutoRP Discovery stopped", __func__);
-	}
+	if (PIM_DEBUG_AUTORP)
+		zlog_debug("%s: AutoRP Discovery stopped", __func__);
+
+	/* Close the socket if we need to */
+	if (pim_autorp_should_close(autorp) && !pim_autorp_socket_disable(autorp))
+		zlog_warn("%s: AutoRP failed to close socket", __func__);
 }
 
 void pim_autorp_init(struct pim_instance *pim)
@@ -1016,14 +1050,12 @@ void pim_autorp_init(struct pim_instance *pim)
 
 	pim->autorp = autorp;
 
-	if (!pim_autorp_socket_enable(autorp)) {
-		zlog_err("%s: AutoRP failed to initialize, feature will not work correctly", __func__);
-		return;
-	}
-
 	if (PIM_DEBUG_AUTORP)
 		zlog_debug("%s: AutoRP Initialized", __func__);
+}
 
+void pim_autorp_enable(struct pim_instance *pim)
+{
 	/* Start AutoRP discovery by default on startup */
 	pim_autorp_start_discovery(pim);
 }
