@@ -276,4 +276,6 @@ void pim_vrf_terminate(void)
 	}
 
 	vrf_terminate();
+	/* Delete the vxlan_info.work_list as all the VRFs are deleted*/
+	pim_vxlan_work_list_delete();
 }
