@@ -19,7 +19,6 @@ import pytest
 from lib.common_config import step
 from lib.micronet import commander
 from lib.topogen import Topogen, TopoRouter
-from lib.topolog import logger
 from lib.topotest import json_cmp
 
 CWD = os.path.dirname(os.path.realpath(__file__))
@@ -60,7 +59,7 @@ def tgen(request):
     tgen.start_topology()
     router_list = tgen.routers()
 
-    for rname, router in router_list.items():
+    for _, router in router_list.items():
         router.load_config(TopoRouter.RD_ZEBRA, "zebra.conf", f"-M grpc:{GRPCP_ZEBRA}")
         router.load_config(TopoRouter.RD_STATIC, "", f"-M grpc:{GRPCP_STATICD}")
         # router.load_config(TopoRouter.RD_BFDD, "", f"-M grpc:{GRPCP_BFDD}")
